package com.jpattern.orm.persistor.statement;

import java.io.InputStream;
import java.io.Reader;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.NClob;
import java.sql.RowId;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Map;

/**
 * 
 * @author ufo
 *
 */
public class PreparedStatementWriterFactory {

	private Map<Class<?>, PreparedStatementWriter<?>> psWriters = new HashMap<Class<?>, PreparedStatementWriter<?>>();
	private PreparedStatementWriter<?> defaultPsWriter = new ObjectPreparedStatementWriter();

	public PreparedStatementWriterFactory() {
		int hello;
		//TODO add writers for primitive types!

		addPsWriter(Blob.class, new BlobPreparedStatementWriter());
		addPsWriter(byte[].class, new ByteArrayPreparedStatementWriter());
		addPsWriter(Clob.class, new ClobPreparedStatementWriter());
		addPsWriter(Date.class, new DatePreparedStatementWriter());
		addPsWriter(InputStream.class, new InputStreamPreparedStatementWriter());
		addPsWriter(NClob.class, new NClobPreparedStatementWriter());
		addPsWriter(Reader.class, new ReaderPreparedStatementWriter());
		addPsWriter(RowId.class, new RowIdPreparedStatementWriter());
		addPsWriter(String.class, new StringPreparedStatementWriter());
		addPsWriter(Time.class, new TimePreparedStatementWriter());
		addPsWriter(Timestamp.class, new TimestampPreparedStatementWriter());

	}

	public <T> PreparedStatementWriter<?> getPreparedStatementWriter(final Class<T> clazz) {
		//TODO add the right type for the returned writer; remove the default writer and throw exception for unknown types.

		if (this.psWriters.containsKey(clazz)) {
			return this.psWriters.get(clazz);
		}
		return this.defaultPsWriter;
	}

	private <T> void addPsWriter(final Class<T> type, final PreparedStatementWriter<T> psWriter) {
		this.psWriters.put(type, psWriter);
	}
}
