package com.jpattern.orm.query.clause;

import java.util.ArrayList;
import java.util.List;

import com.jpattern.orm.classtool.IOrmClassToolMap;
import com.jpattern.orm.query.INameSolver;
import com.jpattern.orm.query.IQueryRoot;
import com.jpattern.orm.query.NullNameSolver;
import com.jpattern.orm.query.from.JoinElement;
import com.jpattern.orm.query.from.FullOuterJoinElement;
import com.jpattern.orm.query.from.FromElement;
import com.jpattern.orm.query.from.InnerJoinElement;
import com.jpattern.orm.query.from.LeftOuterJoinElement;
import com.jpattern.orm.query.from.NaturalJoinElement;
import com.jpattern.orm.query.from.RightOuterJoinElement;

/**
 * 
 * @author Francesco Cina
 *
 * 27/giu/2011
 */
public class OrmFrom<T extends IQueryRoot> implements From<T> {

	private INameSolver nameSolver = new NullNameSolver();
	private final List<FromElement> joinElements = new ArrayList<FromElement>();
	private final IOrmClassToolMap ormClassToolMap;
	private final T query;

	public OrmFrom (final IOrmClassToolMap ormClassToolMap, final T query) {
		this.ormClassToolMap = ormClassToolMap;
		this.query = query;
	}

	@Override
	public From<T> join(final Class<?> joinClass) {
		return join(joinClass, joinClass.getSimpleName());
	}

	@Override
	public From<T> join(final Class<?> joinClass, final String joinClassAlias) {
		Integer nameSolverClassId = this.nameSolver.register(joinClass, joinClassAlias);
		final FromElement joinElement = new JoinElement(this.ormClassToolMap, joinClass, nameSolverClassId);
		joinElement.setNameSolver(this.nameSolver);
		this.joinElements.add(joinElement);
		return this;
	}

	@Override
	public From<T> naturalJoin(final Class<?> joinClass) {
		return naturalJoin(joinClass, joinClass.getSimpleName());
	}

	@Override
	public From<T> naturalJoin(final Class<?> joinClass, final String joinClassAlias) {
		Integer nameSolverClassId = this.nameSolver.register(joinClass, joinClassAlias);
		final FromElement joinElement = new NaturalJoinElement(this.ormClassToolMap, joinClass, nameSolverClassId);
		joinElement.setNameSolver(this.nameSolver);
		this.joinElements.add(joinElement);
		return this;
	}

	@Override
	public From<T> innerJoin(final Class<?> joinClass) {
		return innerJoin(joinClass, joinClass.getSimpleName());
	}

	@Override
	public From<T> innerJoin(final Class<?> joinClass, final String joinClassAlias) {
		Integer nameSolverClassId = this.nameSolver.register(joinClass, joinClassAlias);
		final FromElement joinElement = new InnerJoinElement(this.ormClassToolMap, joinClass, nameSolverClassId);
		joinElement.setNameSolver(this.nameSolver);
		this.joinElements.add(joinElement);
		return this;
	}

	@Override
	public From<T> innerJoin(final Class<?> joinClass, final String onLeftProperty, final String onRigthProperty) {
		return innerJoin(joinClass, joinClass.getSimpleName(), onLeftProperty, onRigthProperty);
	}

	@Override
	public From<T> innerJoin(final Class<?> joinClass, final String joinClassAlias, final String onLeftProperty, final String onRigthProperty) {
		Integer nameSolverClassId = this.nameSolver.register(joinClass, joinClassAlias);
		final FromElement joinElement = new InnerJoinElement(this.ormClassToolMap, joinClass, nameSolverClassId, onLeftProperty, onRigthProperty);
		joinElement.setNameSolver(this.nameSolver);
		this.joinElements.add(joinElement);
		return this;
	}

	@Override
	public From<T> leftOuterJoin(final Class<?> joinClass) {
		return leftOuterJoin(joinClass, joinClass.getSimpleName());
	}

	@Override
	public From<T> leftOuterJoin(final Class<?> joinClass, final String joinClassAlias) {
		Integer nameSolverClassId = this.nameSolver.register(joinClass, joinClassAlias);
		final FromElement joinElement = new LeftOuterJoinElement(this.ormClassToolMap, joinClass, nameSolverClassId);
		joinElement.setNameSolver(this.nameSolver);
		this.joinElements.add(joinElement);
		return this;
	}

	@Override
	public From<T> leftOuterJoin(final Class<?> joinClass, final String onLeftProperty, final String onRigthProperty) {
		return leftOuterJoin(joinClass, joinClass.getSimpleName(), onLeftProperty, onRigthProperty);
	}

	@Override
	public From<T> leftOuterJoin(final Class<?> joinClass, final String joinClassAlias, final String onLeftProperty, final String onRigthProperty) {
		Integer nameSolverClassId = this.nameSolver.register(joinClass, joinClassAlias);
		final FromElement joinElement = new LeftOuterJoinElement(this.ormClassToolMap, joinClass, nameSolverClassId, onLeftProperty, onRigthProperty);
		joinElement.setNameSolver(this.nameSolver);
		this.joinElements.add(joinElement);
		return this;
	}

	@Override
	public From<T> rightOuterJoin(final Class<?> joinClass) {
		return rightOuterJoin(joinClass, joinClass.getSimpleName());
	}

	@Override
	public From<T> rightOuterJoin(final Class<?> joinClass, final String joinClassAlias) {
		Integer nameSolverClassId = this.nameSolver.register(joinClass, joinClassAlias);
		final FromElement joinElement = new RightOuterJoinElement(this.ormClassToolMap, joinClass, nameSolverClassId);
		joinElement.setNameSolver(this.nameSolver);
		this.joinElements.add(joinElement);
		return this;
	}

	@Override
	public From<T> rightOuterJoin(final Class<?> joinClass, final String onLeftProperty, final String onRigthProperty) {
		return rightOuterJoin(joinClass, joinClass.getSimpleName(), onLeftProperty, onRigthProperty);
	}

	@Override
	public From<T> rightOuterJoin(final Class<?> joinClass, final String joinClassAlias, final String onLeftProperty, final String onRigthProperty) {
		Integer nameSolverClassId = this.nameSolver.register(joinClass, joinClassAlias);
		final FromElement joinElement = new RightOuterJoinElement(this.ormClassToolMap, joinClass, nameSolverClassId, onLeftProperty, onRigthProperty);
		joinElement.setNameSolver(this.nameSolver);
		this.joinElements.add(joinElement);
		return this;
	}

	@Override
	public From<T> fullOuterJoin(final Class<?> joinClass) {
		return fullOuterJoin(joinClass, joinClass.getSimpleName());
	}

	@Override
	public From<T> fullOuterJoin(final Class<?> joinClass, final String joinClassAlias) {
		Integer nameSolverClassId = this.nameSolver.register(joinClass, joinClassAlias);
		final FromElement joinElement = new FullOuterJoinElement(this.ormClassToolMap, joinClass, nameSolverClassId);
		joinElement.setNameSolver(this.nameSolver);
		this.joinElements.add(joinElement);
		return this;
	}

	@Override
	public From<T> fullOuterJoin(final Class<?> joinClass, final String onLeftProperty, final String onRigthProperty) {
		return fullOuterJoin(joinClass, joinClass.getSimpleName(), onLeftProperty, onRigthProperty);
	}

	@Override
	public From<T> fullOuterJoin(final Class<?> joinClass, final String joinClassAlias, final String onLeftProperty, final String onRigthProperty) {
		Integer nameSolverClassId = this.nameSolver.register(joinClass, joinClassAlias);
		final FromElement joinElement = new FullOuterJoinElement(this.ormClassToolMap, joinClass, nameSolverClassId, onLeftProperty, onRigthProperty);
		joinElement.setNameSolver(this.nameSolver);
		this.joinElements.add(joinElement);
		return this;
	}

	@Override
	public void setNameSolver(final INameSolver nameSolver) {
		this.nameSolver = nameSolver;
	}

	@Override
	public String renderSql() {
		final StringBuilder query = new StringBuilder();
		renderSql(query);
		return query.toString();
	}

	@Override
	public void renderSql(final StringBuilder StringBuilder) {
		for (final FromElement joinElement : this.joinElements) {
			joinElement.renderSql(StringBuilder);
		}
	}

	@Override
	public T query() {
		return this.query;
	}

}
